## SISCER 2021: Generalized Estimating Equations for Longitudinal Data Analysis
## Benjamin French; b.french@vumc.org

install.packages(c('geepack', 'dplyr', 'ggplot2', 'JM'), 
                 repos='https://cloud.r-project.org')

library(geepack)
library(dplyr)
library(ggplot2)
library(JM)

theme_set(theme_bw())
mytheme <- function(...){
  theme(panel.grid.minor=element_blank(),
        plot.margin=unit(c(0.5, 0.5, 0.5, 0.5), 'lines'),
        axis.title.x=element_text(size=12, vjust=0),
        axis.title.y=element_text(size=12, vjust=1, angle=90),
        legend.title=element_text(size=12, face='bold'),
        legend.text=element_text(size=12),
        legend.position='top', legend.direction='horizontal', 
        legend.key=element_blank(),
        legend.key.width=unit(1.5,"cm"),
        legend.key.height=unit(0.35,'cm'),
        strip.text.y=element_text(size=12))}

?pbc2
data(pbc2)
head(pbc2, 11)
str(pbc2)

pbc2 %>%
  group_by(id) %>%
  summarize(m=sum(!is.na(albumin))) %>%
  summarize(min(m), median(m), max(m))

subset(pbc2, !is.na(albumin)) %>% 
  count(id, name='m') -> num
with(num, table(m))

ggplot(subset(pbc2, is.element(id, 1:16)), aes(year, albumin)) +
  facet_wrap( ~ id) +
  geom_point() +
  geom_smooth(se=FALSE, method='lm') + 
  scale_x_continuous(name='Time, years', limits=c(0,15)) +
  scale_y_continuous(name='Serum albumin, g/dl') + 
  mytheme()

?geom_smooth

pbc2 <- within(pbc2, {
  levels(drug) <- c('Placebo', 'Treatment')})

ggplot(subset(pbc2, albumin<6), aes(year, albumin, group=id)) +
  geom_line(aes(color=drug), alpha=0.4) +
  geom_smooth(aes(year, albumin, group=drug, color=drug), se=FALSE, size=1.5) +
  scale_x_continuous(name='Time, years', limits=c(0,15)) +
  scale_y_continuous(name='Serum albumin, g/dl') + 
  scale_color_manual(name='Intervention', values=c('#00BFC4', '#F8766D')) + 
  mytheme()

pbc2 %>% 
  group_by(id) %>% 
  mutate(obs_num = row_number()) %>%
  ungroup() %>%
  as.data.frame() -> pbc2

pbc2_wide <- reshape(subset(pbc2, select=c(id, obs_num, albumin)),
                     v.names='albumin', idvar='id', timevar='obs_num', direction='wide')
head(pbc2_wide, 2)

ntime <- 8
rmat <- with(pbc2_wide, cbind(albumin.1, albumin.2, albumin.3, albumin.4, albumin.5, albumin.6, albumin.7, albumin.8))

vmat <- nmat <- covmatj <- covmatk <- matrix(0, nrow=ntime, ncol=ntime)
rownames(vmat) <- colnames(vmat) <- 1:ntime
rownames(nmat) <- colnames(nmat) <- 1:ntime

for(j in 1:ntime){
  for(k in 1:ntime){
    nmat[j,k] <- sum(!is.na(rmat[,j]*rmat[,k]))
    vmat[j,k] <- cov(rmat[,j],rmat[,k],use="pairwise.complete.obs")
    if(nmat[j,k]>0) covmatj[j,k] <- cov(rmat[,j][!is.na(rmat[,j]*rmat[,k])],rmat[,j][!is.na(rmat[,j]*rmat[,k])],use="pairwise.complete.obs")
    if(nmat[j,k]>0) covmatk[j,k] <- cov(rmat[,k][!is.na(rmat[,j]*rmat[,k])],rmat[,k][!is.na(rmat[,j]*rmat[,k])],use="pairwise.complete.obs")
  }}

cmat <- vmat/sqrt(covmatj)/sqrt(covmatk)

round(diag(vmat),2)
round(cmat, 2)
nmat

?geeglm

# Main effects of treatment and time, independence correlation
gee_m1 <- geeglm(formula=albumin ~ drug + year, family='gaussian', 
                 data=pbc2, id=id, corstr='independence', scale.fix=TRUE)
summary(gee_m1)

# Main effects of treatment and time, exchangeable correlation
gee_m2 <- geeglm(formula=albumin ~ drug + year,  family='gaussian', 
                 data=pbc2, id=id, corstr='exchangeable', scale.fix=TRUE)
summary(gee_m2)

# Interaction between treatment and time, exchangeable correlation
gee_m3 <- geeglm(formula=albumin ~ drug*year, family='gaussian', 
                 data=pbc2, id=id, corstr='exchangeable', scale.fix=TRUE)
summary(gee_m3)

# Treatment-specific time effects, exchangeable correlation
gee_m4 <- geeglm(formula=albumin ~ drug:year, family='gaussian', 
                 data=pbc2, id=id, corstr='exchangeable', scale.fix=TRUE)
summary(gee_m4)

# 95% confidence intervals
broom::tidy(gee_m4, conf.int=TRUE)

plot(pbc2$year, residuals(gee_m4), xlab='Time, years', ylab='Residuals')
abline(h=0, col='gray', lwd=2)
lines(lowess(pbc2$year, residuals(gee_m4)), col='red', lwd=2)

gee_m5 <- geeglm(formula=albumin ~ drug:year + drug:I(year^2), family='gaussian',
                 data=pbc2, id=id, corstr='exchangeable', scale.fix=TRUE)
anova(gee_m5)

plot(pbc2$year, residuals(gee_m5), xlab='Time, years', ylab='Residuals')
abline(h=0, col='gray', lwd=2)
lines(lowess(pbc2$year, residuals(gee_m5)), col='red', lwd=2)

print(sessionInfo(), locale = FALSE)
